@@ -7,6 +7,7 @@ import numpy as np
 import pickle
 import numpy as numpy
 import os
+from ingradient_library.patch_transform import Batch_Affine_3D
 
 
 class CustomDataset(Dataset):
@@ -40,6 +41,10 @@ class CustomDataset(Dataset):
         return len(self.image_name)
     
     def set_direction(self, img, direction):
+        if isinstance(img, np.ndarray):
+            img = torch.tensor(img)
+        img = img.permute(2,1,0)
+        img = img.numpy()
         direction = np.array(direction).reshape(3,3)
         inverse = np.where(direction == -1)[0]
         permute = np.where(direction != 0)[1]
@@ -53,7 +58,7 @@ class CustomDataset(Dataset):
         if 2 in inverse:
             img = img[:, :, ::-1]
 
-        return img.transpose(permute)
+        return torch.tensor(img.copy())
             
     def __getitem__(self, idx):
         info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
@@ -72,8 +77,11 @@ class CustomDataset(Dataset):
 
         if self.correct_direction:
             for i in range(len(x)):
-                x[i] = self.set_direction(x[i], info_data['direction'])
-            
+                if i == 0:
+                    result = self.set_direction(x[i], info_data['direction']).unsqueeze(0)
+                else:
+                    result = torch.vstack((result, self.set_direction(x[i], info_data['direction']).unsqueeze(0)))
+            x = result 
             if self.mode == 'train' :
                 y = self.set_direction(y, info_data['direction'])
 
@@ -83,7 +91,8 @@ class CustomDataset(Dataset):
         
         if self.resample != None:
             if self.mode == 'train' :
-                y = torch.tensor(y.copy())
+                if isinstance(y, np.ndarray):
+                    y = torch.tensor(y.copy())
                 result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
                 x = result[:-1]
                 y = result[-1].squeeze(0)
@@ -121,202 +130,45 @@ class CustomDataset(Dataset):
                 pickle_file.close()
             
 
-
-class CustomDatasetV2(CustomDataset):
-    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None,
-                 patch_size = (128, 128, 128), transform = None, batch_size = 2):
-        
-        self.patch_size = np.array(patch_size)
-        self.transform = transform
-        self.batch_size = batch_size
-        super().__init__(root_dir, device = device, normalizer = normalizer, mode = mode, dim = dim, correct_direction = correct_direction, resample = resample)
-    
-    def __len__(self):
-        return super().__len__()
+class Dataset2point5(CustomDataset):
+    def __init__(self, root_dir, degree, device=None, normalizer=None, mode='train', dim='3d', correct_direction=True, resample=None):
+        super().__init__(root_dir, device=device, normalizer=normalizer, mode=mode, dim=dim, correct_direction=correct_direction, resample=resample)
+        self.transform = Batch_Affine_3D([degree], [0], is_random = False)
     
     def set_direction(self, img, direction):
         return super().set_direction(img, direction)
     
-    def get_oversample_patch(self, images, seg):
-        if not isinstance(images, np.ndarray):
-            images = images.numpy()
-
-        if not isinstance(seg, np.ndarray):
-            seg = seg.numpy()
-
-        non_zero_coords = np.array(np.where(seg != 0))
-        shape_to_index = np.array(images[0].shape) - 1 
-        
-        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
-        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
-        oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
-
-        oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
-        oversample_patch_lower_bound = oversample_center - (self.patch_size/2 - 1)
-        oversample_patch_bound = np.concatenate((oversample_patch_lower_bound.reshape(-1,1),oversample_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
-
-        oversample_patch_images = images[:, oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
-                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
-                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
-        oversample_patch_seg = seg[oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
-                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
-                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
-        
-        return np.expand_dims(oversample_patch_images , axis = 0), np.expand_dims(oversample_patch_seg , axis = 0)
-    
-    def get_normal_patch(self, images, seg):
-        if not isinstance(images, np.ndarray):
-            images = images.numpy()
-
-        if not isinstance(seg, np.ndarray):
-            seg = seg.numpy()
-
-        shape_to_index = np.array(images[0].shape) - 1 
-        normal_center = np.random.randint(low = self.patch_size/2 - 1, high = shape_to_index - (self.patch_size/2) + 1 )
-        normal_patch_upper_bound = normal_center + (self.patch_size/2) + 1
-        normal_patch_lower_bound = normal_center - (self.patch_size/2 - 1)
-        normal_patch_bound = np.concatenate((normal_patch_lower_bound.reshape(-1,1),normal_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
-        normal_patch_images = images[:, normal_patch_bound[0][0] : normal_patch_bound[0][1],
-                                          normal_patch_bound[1][0] : normal_patch_bound[1][1],
-                                          normal_patch_bound[2][0] : normal_patch_bound[2][1]]
-        normal_patch_seg = seg[normal_patch_bound[0][0] : normal_patch_bound[0][1],
-                                          normal_patch_bound[1][0] : normal_patch_bound[1][1],
-                                          normal_patch_bound[2][0] : normal_patch_bound[2][1]]
-        
-        return np.expand_dims(normal_patch_images , axis = 0), np.expand_dims(normal_patch_seg , axis = 0)
-
-    """
-    def get_oversample_patch(self, images, seg):
-        non_zero_coords = torch.where(seg != 0)
-        shape_to_index = torch.tensor(images[0].shape) - 1
-        oversample_max_index = torch.max(torch.vstack(non_zero_coords), axis = 1)[0][:-3] #class가 있는 부분의 최대 index
-        oversample_min_index = torch.min(torch.vstack(non_zero_coords), axis = 1)[0][:-3] #class가 있는 부분의 최소 index
-        self.front_patch_size = self.patch_size//2
-        self.back_patch_size = shape_to_index - (self.patch_size//2 + self.patch_size%2)
-
-        oversample_center_upper_bound = torch.min((oversample_max_index, self.back_patch_size))
-    
-        oversample_center_x = torch.randint(low = oversample_min_index[-3], high = oversample_max_index[-3]) 
-        oversample_center_y = torch.randint(low = oversample_min_index[-2], high = oversample_max_index[-2])
-        oversample_center_z = torch.randint(low = oversample_min_index[-1], high = oversample_max_index[-1])
-
-        oversample_high_x = self.patch_size 
-        
-        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
-        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
-        oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
-
-        oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
-        oversample_patch_lower_bound = oversample_center - (self.patch_size/2 - 1)
-        oversample_patch_bound = np.concatenate((oversample_patch_lower_bound.reshape(-1,1),oversample_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
-
-        oversample_patch_images = images[:, oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
-                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
-                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
-        oversample_patch_seg = seg[oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
-                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
-                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
-        
-        return np.expand_dims(oversample_patch_images , axis = 0), np.expand_dims(oversample_patch_seg , axis = 0)
-    """
-    
-    def pad_image_seg(self, images, seg): #patch size보다 전체 이미지 크기가 작을 경우 padding을 진행
-        if isinstance(images, np.ndarray):
-            images = torch.tensor(images.copy())
-        if isinstance(seg, np.ndarray):
-            seg = torch.tensor(seg.copy())
-        image_shape = np.array(images[0].shape)
-        if np.any(image_shape < self.patch_size):
-            odd_pad = ((self.patch_size - image_shape) % 2 != 0).astype(int)
-            even_pad = np.repeat(np.clip(((self.patch_size - image_shape)/2).astype(int), 0, np.inf), 2)
-            for i in range(len(odd_pad)):
-                even_pad[i*2] += odd_pad[i]
-            even_pad = tuple(even_pad[::-1].astype(int).copy())
-            images = F.pad(images, even_pad, "constant", 0 )
-            seg = F.pad(seg, even_pad, "constant", 0 )
-
-        return images, seg
+    def __len__(self):
+        return super().__len__()
 
     def __getitem__(self, idx):
-        info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
-        info_data = pickle.load(info_file)
-        info_file.close()
-        data = np.load(os.path.join(self.root_dir, self.image_name[idx]))
+        images, seg, info_data = super().__getitem__(idx) # images => n_modalities, x, y, z, seg도 마찬가지
         
-
-        x = data['x']
-        y = data['y']
-        if len(x.shape) == 3 and self.dim == '3d':
-            #modality가 1개인 경우
-            x = np.expand_dims(x, axis = 0)
-        elif len(x.shape) == 2 and self.dim == '2d':
-            x = np.expand_dims(x, axis = 0)
-
-        if self.correct_direction:
-            for i in range(len(x)):
-                x[i] = self.set_direction(x[i], info_data['direction'])
-            
-            if self.mode == 'train' :
-                y = self.set_direction(y, info_data['direction'])
-
-        if self.normalizer:
-            x = self.normalizer(x)
-        
-        
-        if self.resample != None:
-            if self.mode == 'train' :
-                y = torch.tensor(y.copy())
-                result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
-                x = result[:-1]
-                y = result[-1].squeeze(0)
-            
+        if self.transform != None:
+            if isinstance(images, np.ndarray):
+                images = np.expand_dims(images, aixs = 0)
             else:
-                x = self.resample(x, info_data)
-        
-        x, y = self.pad_image_seg(x, y)
-        if self.batch_size == 1 :
-            result_images, result_seg = self.get_normal_patch(x, y)
+                images = images.unsqueeze(0)
 
-
-        for i_b in range(self.batch_size//2):
-            oversample_images, oversample_seg = self.get_oversample_patch(x, y)
-            normal_images, normal_seg = self.get_normal_patch(x, y)
-
-            if i_b == 0:
-                result_images = np.concatenate((normal_images, oversample_images), axis = 0) #bs, n, x, y, z
-                result_seg = np.concatenate((normal_seg, oversample_seg), axis = 0) #bs, x, y, z
-            
+            if isinstance(seg, np.ndarray):
+                seg = np.expand_dims(seg, axis = 0)
             else:
-                result_images = np.concatenate((result_images, oversample_images), axis = 0)
-                result_seg = np.concatenate((result_seg, oversample_seg), axis = 0)
-                result_images = np.concatenate((result_images, normal_images), axis = 0)
-                result_seg = np.concatenate((result_seg, normal_seg), axis = 0)
+                seg = seg.unsqueeze(0)
+
+            images = self.transform(images)
+            seg = self.transform(seg)
+            while len(images.shape) > 4:
+                images = images.squeeze(0)
+            while len(seg.shape) > 3:
+                seg = seg.squeeze(0)
 
 
-        if isinstance(result_images, np.ndarray):
-            result_images = torch.tensor(result_images)
-        
-        if isinstance(result_seg, np.ndarray):
-            result_seg = torch.tensor(result_seg)
-        
-        if result_images.device.index != self.device:
-            result_images = result_images.to(self.device)
-
-        if result_seg.device.index != self.device:
-            result_seg = result_seg.to(self.device)
-
-        if self.transform:
-            result_images, result_seg = self.transform(result_images.to(self.device), result_seg.to(self.device), info_data)
-        
-
-        return result_images, result_seg.long()
-
+        return images, seg
     
 
 
 
 class DataLoader3D(object):
-
     def __init__(self, dataset, num_iteration = 2, patch_size = (480,256,256), transform= None, device = 0, batch_size = 2, seg_one_hot = False, is_half = False, info_class = None):
         self.dataset = dataset
         self.current_index = 0
@@ -358,31 +210,29 @@ class DataLoader3D(object):
             images = torch.tensor(images.copy())
         if isinstance(seg, np.ndarray):
             seg = torch.tensor(seg.copy())
-        image_shape = np.array(images[0].shape)
+        image_shape = np.array(images[0].shape)[-3:]
         if np.any(image_shape < self.patch_size):
-            odd_pad = (np.max(0,(self.patch_size - image_shape), axis = 1) % 2 != 0).astype(int)
+            odd_pad = (np.max((np.array([0,0,0]),(self.patch_size - image_shape)), axis = 0) % 2 != 0).astype(int)
             even_pad = np.repeat(np.clip(((self.patch_size - image_shape)/2).astype(int), 0, np.inf), 2)
             for i in range(len(odd_pad)):
                 even_pad[i*2] += odd_pad[i]
             even_pad = tuple(even_pad[::-1].astype(int).copy())
-            print("even_pad", even_pad)
-            print("before images shape", images.shape, "seg shape", seg.shape)
             images = F.pad(images, even_pad, "constant", 0 )
             seg = F.pad(seg, even_pad, "constant", 0 )
-            print("after images shape", images.shape, "seg shape", seg.shape)
 
         return images, seg, info
 
 
     def get_oversample_patch(self, non_zero_coords, shape_to_index):
-        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
-        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
-        oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
+        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2).reshape(1,3) + 1
+        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2).reshape(1,3)
+        oversample_high = oversample_high.astype(int)
+        oversample_low = oversample_low.astype(int)
+        oversample_center = np.random.randint(oversample_low, oversample_high)
 
         oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
         oversample_patch_lower_bound = oversample_center - (self.patch_size/2 - 1)
         oversample_patch_bound = np.concatenate((oversample_patch_lower_bound.reshape(-1,1),oversample_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
-
         oversample_patch_images = self.images[:, oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
                                           oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
                                           oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
@@ -452,6 +302,86 @@ class DataLoader3D(object):
         return result_images, result_seg.long()
 
 
+"""
+class DataLoader2D(object):
+    def __init__(self, dataset, batch_size = 16, full_volume_size = (600, 600, 720), transform = None):
+        self.current_index = 0
+        self.images, self.seg, self.info = 
+        self.transform = transform
+        self.current_batch = 0
+        self.batch_size = batch_size
+    
+    def div_batch(self):
+        # batch를 배수에 맞게 나눈다.
+        # mat_batch를 정한다.
+        
+    
+    def next_index(self):
+        if self.current_batch == self.n_batch:
+            self.images, self.seg, self.info = self.get_image_seg(self.current_index)
+            self.current_index += 1
+            self.now_iter = 0
+        
+        else:
+            self.now_iter += 1
+
+    def get_image_seg(self, current_index): #patch size보다 전체 이미지 크기가 작을 경우 padding을 진행
+        images, seg, info = self.dataset[current_index]
+        if isinstance(images, np.ndarray):
+            images = torch.tensor(images.copy())
+        if isinstance(seg, np.ndarray):
+            seg = torch.tensor(seg.copy())
+        image_shape = np.array(images[0].shape)[-3:]
+        if np.any(image_shape < self.patch_size):
+            odd_pad = (np.max((np.array([0,0,0]),(self.patch_size - image_shape)), axis = 0) % 2 != 0).astype(int)
+            even_pad = np.repeat(np.clip(((self.patch_size - image_shape)/2).astype(int), 0, np.inf), 2)
+            for i in range(len(odd_pad)):
+                even_pad[i*2] += odd_pad[i]
+            even_pad = tuple(even_pad[::-1].astype(int).copy())
+            images = F.pad(images, even_pad, "constant", 0 )
+            seg = F.pad(seg, even_pad, "constant", 0 )
+        if len(images.shape)
+
+        return images, seg, info
+    
+    def sampling_by_axis(self):
+        # 축을 따라 sampling
+
+        return
+    
+
+    def new_epoch(self):
+        self.now_iter = 0
+        self.current_index =
+
+    def padding(self, image, seg):
+        # 전체 Image volume을 가져올 때 padding을 진행한다.
+
+    def get_image_seg(self, current_index):
+        return super().get_image_seg(current_index) 0
+    
+    def next_index(self):
+        if self.now_iter == self.num_iteration:
+            self.images, self.seg, self.info = self.get_image_seg(self.current_index)
+            self.current_index += 1
+            self.now_iter = 0
+        
+        else:
+            self.now_iter += 1
+
+    def cal_batch(self):
+        
+
+    def generate_train_batch(self):
+
+
+        
+        if self.transform:
+            result_images, result_seg = self.transform(result_images, result_seg, self.info)
+
+"""
+
+
 
 class Interactive_DataLoader(DataLoader3D):
     def __init__(self, dataset, num_iteration = 1, patch_size = (480,256,256), batch_size = 2, device = None, transform = None):
@@ -544,4 +474,191 @@ class Interactive_DataLoader(DataLoader3D):
 
         return new_images, new_seg
     
-    
\ No newline at end of file
+
+
+
+
+class CustomDatasetV2(CustomDataset):
+    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None,
+                 patch_size = (128, 128, 128), transform = None, batch_size = 2):
+        
+        self.patch_size = np.array(patch_size)
+        self.transform = transform
+        self.batch_size = batch_size
+        super().__init__(root_dir, device = device, normalizer = normalizer, mode = mode, dim = dim, correct_direction = correct_direction, resample = resample)
+    
+    def __len__(self):
+        return super().__len__()
+    
+    def set_direction(self, img, direction):
+        return super().set_direction(img, direction)
+    
+    def get_oversample_patch(self, images, seg):
+        if not isinstance(images, np.ndarray):
+            images = images.numpy()
+
+        if not isinstance(seg, np.ndarray):
+            seg = seg.numpy()
+
+        non_zero_coords = np.array(np.where(seg != 0))
+        shape_to_index = np.array(images[0].shape) - 1 
+        
+        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+        oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
+
+        oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
+        oversample_patch_lower_bound = oversample_center - (self.patch_size/2 - 1)
+        oversample_patch_bound = np.concatenate((oversample_patch_lower_bound.reshape(-1,1),oversample_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
+
+        oversample_patch_images = images[:, oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
+                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
+                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
+        oversample_patch_seg = seg[oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
+                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
+                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
+        
+        return np.expand_dims(oversample_patch_images , axis = 0), np.expand_dims(oversample_patch_seg , axis = 0)
+    
+    def get_normal_patch(self, images, seg):
+        if not isinstance(images, np.ndarray):
+            images = images.numpy()
+
+        if not isinstance(seg, np.ndarray):
+            seg = seg.numpy()
+
+        shape_to_index = np.array(images[0].shape) - 1 
+        normal_center = np.random.randint(low = self.patch_size/2 - 1, high = shape_to_index - (self.patch_size/2) + 1 )
+        normal_patch_upper_bound = normal_center + (self.patch_size/2) + 1
+        normal_patch_lower_bound = normal_center - (self.patch_size/2 - 1)
+        normal_patch_bound = np.concatenate((normal_patch_lower_bound.reshape(-1,1),normal_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
+        normal_patch_images = images[:, normal_patch_bound[0][0] : normal_patch_bound[0][1],
+                                          normal_patch_bound[1][0] : normal_patch_bound[1][1],
+                                          normal_patch_bound[2][0] : normal_patch_bound[2][1]]
+        normal_patch_seg = seg[normal_patch_bound[0][0] : normal_patch_bound[0][1],
+                                          normal_patch_bound[1][0] : normal_patch_bound[1][1],
+                                          normal_patch_bound[2][0] : normal_patch_bound[2][1]]
+        
+        return np.expand_dims(normal_patch_images , axis = 0), np.expand_dims(normal_patch_seg , axis = 0)
+
+    
+    def pad_image_seg(self, images, seg): #patch size보다 전체 이미지 크기가 작을 경우 padding을 진행
+        if isinstance(images, np.ndarray):
+            images = torch.tensor(images.copy())
+        if isinstance(seg, np.ndarray):
+            seg = torch.tensor(seg.copy())
+        image_shape = np.array(images[0].shape)
+        if np.any(image_shape < self.patch_size):
+            odd_pad = ((self.patch_size - image_shape) % 2 != 0).astype(int)
+            even_pad = np.repeat(np.clip(((self.patch_size - image_shape)/2).astype(int), 0, np.inf), 2)
+            for i in range(len(odd_pad)):
+                even_pad[i*2] += odd_pad[i]
+            even_pad = tuple(even_pad[::-1].astype(int).copy())
+            images = F.pad(images, even_pad, "constant", 0 )
+            seg = F.pad(seg, even_pad, "constant", 0 )
+
+        return images, seg
+
+    def __getitem__(self, idx):
+        info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
+        info_data = pickle.load(info_file)
+        info_file.close()
+        data = np.load(os.path.join(self.root_dir, self.image_name[idx]))
+        
+
+        x = data['x']
+        y = data['y']
+        if len(x.shape) == 3 and self.dim == '3d':
+            #modality가 1개인 경우
+            x = np.expand_dims(x, axis = 0)
+        elif len(x.shape) == 2 and self.dim == '2d':
+            x = np.expand_dims(x, axis = 0)
+
+        if self.correct_direction:
+            for i in range(len(x)):
+                x[i] = self.set_direction(x[i], info_data['direction'])
+            
+            if self.mode == 'train' :
+                y = self.set_direction(y, info_data['direction'])
+
+        if self.normalizer:
+            x = self.normalizer(x)
+        
+        
+        if self.resample != None:
+            if self.mode == 'train' :
+                y = torch.tensor(y.copy())
+                result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
+                x = result[:-1]
+                y = result[-1].squeeze(0)
+            
+            else:
+                x = self.resample(x, info_data)
+        
+        x, y = self.pad_image_seg(x, y)
+        if self.batch_size == 1 :
+            result_images, result_seg = self.get_normal_patch(x, y)
+
+
+        for i_b in range(self.batch_size//2):
+            oversample_images, oversample_seg = self.get_oversample_patch(x, y)
+            normal_images, normal_seg = self.get_normal_patch(x, y)
+
+            if i_b == 0:
+                result_images = np.concatenate((normal_images, oversample_images), axis = 0) #bs, n, x, y, z
+                result_seg = np.concatenate((normal_seg, oversample_seg), axis = 0) #bs, x, y, z
+            
+            else:
+                result_images = np.concatenate((result_images, oversample_images), axis = 0)
+                result_seg = np.concatenate((result_seg, oversample_seg), axis = 0)
+                result_images = np.concatenate((result_images, normal_images), axis = 0)
+                result_seg = np.concatenate((result_seg, normal_seg), axis = 0)
+
+
+        if isinstance(result_images, np.ndarray):
+            result_images = torch.tensor(result_images)
+        
+        if isinstance(result_seg, np.ndarray):
+            result_seg = torch.tensor(result_seg)
+        
+        if result_images.device.index != self.device:
+            result_images = result_images.to(self.device)
+
+        if result_seg.device.index != self.device:
+            result_seg = result_seg.to(self.device)
+
+        if self.transform:
+            result_images, result_seg = self.transform(result_images.to(self.device), result_seg.to(self.device), info_data)
+        
+
+        return result_images, result_seg.long()
+
+    
+
+"""
+class DataLoader2D(object):
+    def __init__(self, patch_size = , batch_size = 8, ):
+
+
+    def new_epoch(self):
+        self.now_iter = 0
+        self.current_index = 0
+    
+    def is_end(self):
+        return self.current_index == len(self.dataset) and self.now_iter == 0
+    
+    def return_class(self):
+        return np.unique(self.seg)
+
+    def next_index(self):
+        if self.now_iter == self.num_iteration:
+            self.images, self.seg, self.info = self.get_image_seg(self.current_index)
+            self.current_index += 1
+            self.now_iter = 0
+        
+        else:
+            self.now_iter += 1
+
+    def generate_train_batch(self):
+"""
+
